

class GeneralStatus(object):
    battery_trouble = False
    ac_trouble = False
    bell_trouble = False


class PartitionStatus(object):
    ready = False
    alarm = False
    trouble = False
    armed_away = False
    armed_stay = False
    exit_delay = False
    entry_delay = False


class Partition(object):
    index = None
    status = None

    def __init__(self, index):
        self.index = index
        self.status = PartitionStatus()


class ZoneStatus(object):
    open = False
    fault = False
    alarm = False
    tamper = False


class Zone(object):
    index = None
    status = None

    def __init__(self, index):
        self.index = index
        self.status = ZoneStatus()


class AlarmState(object):
    """
    State of the alarm system.
    """

    def __init__(self):
        self.status = GeneralStatus()
        self.partitions = {}
        self.zones = {}

    def _update(self, item, update):
        for key, value in update.items():
            if isinstance(value, dict):
                self._update(getattr(item, key), value)
            else:
                setattr(item, key, value)

    def update_general(self, update):
        self._update(self.status, update)
        return self.status

    def update_zone(self, zone, update):
        zone = self.get_zone(zone)
        self._update(zone, update)
        return zone

    def update_partition(self, partition, update):
        partition = self.get_partition(partition)
        self._update(partition, update)
        return partition

    def get_zone(self, index):
        if index not in self.zones:
            self.zones[index] = Zone(index)

        return self.zones[index]

    def get_partition(self, index):
        if index not in self.partitions:
            self.partitions[index] = Partition(index)

        return self.partitions[index]



import asyncio
import logging

import serial_asyncio

from . import protocol, exceptions, state

try:
    from asyncio import ensure_future
except ImportError:
    # Python 3.4.3 and earlier has this as async
    # pylint: disable=unused-import
    from asyncio import async
    ensure_future = async

# Exports.
__all__ = [
    'Driver'
]

logger = logging.getLogger(__name__)


class Driver(object):
    """
    DSC IT-100 driver.
    """

    # Update types.
    UPDATE_ZONE = 'zone'
    UPDATE_PARTITION = 'partition'
    UPDATE_GENERAL = 'general'

    # State update handlers.
    state_update_handlers = {
        # Zone updates.
        protocol.NOTIFY_ZONE_ALARM: {'type': UPDATE_ZONE, 'status': {'alarm': True}},
        protocol.NOTIFY_ZONE_ALARM_RESTORE: {'type': UPDATE_ZONE, 'status': {'alarm': False}},
        protocol.NOTIFY_ZONE_TAMPER: {'type': UPDATE_ZONE, 'status': {'tamper': True}},
        protocol.NOTIFY_ZONE_TAMPER_RESTORE: {'type': UPDATE_ZONE, 'status': {'tamper': False}},
        protocol.NOTIFY_ZONE_FAULT: {'type': UPDATE_ZONE, 'status': {'fault': True}},
        protocol.NOTIFY_ZONE_FAULT_RESTORE: {'type': UPDATE_ZONE, 'status': {'fault': False}},
        protocol.NOTIFY_ZONE_OPEN: {'type': UPDATE_ZONE, 'status': {'open': True}},
        protocol.NOTIFY_ZONE_RESTORED: {'type': UPDATE_ZONE, 'status': {'open': False}},

        # Partition updates.
        protocol.NOTIFY_PARTITION_READY: {'type': UPDATE_PARTITION, 'status': {'ready': True}},
        protocol.NOTIFY_PARTITION_NOT_READY: {'type': UPDATE_PARTITION, 'status': {'ready': False}},
        protocol.NOTIFY_PARTITION_ARMED: {'type': UPDATE_PARTITION},
        protocol.NOTIFY_PARTITION_READY_TO_FORCE_ARM: {'type': UPDATE_PARTITION, 'status': {'ready': True}},
        protocol.NOTIFY_PARTITION_IN_ALARM: {'type': UPDATE_PARTITION, 'status': {'alarm': True}},
        protocol.NOTIFY_PARTITION_DISARMED: {'type': UPDATE_PARTITION, 'status': {
            'alarm': False,
            'armed_stay': False,
            'armed_away': False,
            'exit_delay': False,
            'entry_delay': False,
        }},
        protocol.NOTIFY_PARTITION_EXIT_DELAY: {'type': UPDATE_PARTITION, 'status': {'exit_delay': True}},
        protocol.NOTIFY_PARTITION_ENTRY_DELAY: {'type': UPDATE_PARTITION, 'status': {'entry_delay': True}},
        protocol.NOTIFY_PARTITION_BUSY: {'type': UPDATE_PARTITION},
        protocol.NOTIFY_PARTITION_USER_CLOSING: {'type': UPDATE_PARTITION},
        protocol.NOTIFY_PARTITION_SPECIAL_CLOSING: {'type': UPDATE_PARTITION},
        protocol.NOTIFY_PARTITION_PARTIAL_CLOSING: {'type': UPDATE_PARTITION},
        protocol.NOTIFY_PARTITION_USER_OPENING: {'type': UPDATE_PARTITION, 'status': {
            'alarm': False,
            'armed_stay': False,
            'armed_away': False,
            'exit_delay': False,
            'entry_delay': False,
        }},
        protocol.NOTIFY_PARTITION_SPECIAL_OPENING: {'type': UPDATE_PARTITION, 'status': {
            'alarm': False,
            'armed_stay': False,
            'armed_away': False,
            'exit_delay': False,
            'entry_delay': False,
        }},
        protocol.NOTIFY_PARTITION_TROUBLE: {'type': UPDATE_PARTITION, 'status': {'trouble': True}},
        protocol.NOTIFY_PARTITION_TROUBLE_RESTORED: {'type': UPDATE_PARTITION, 'status': {'trouble': False}},

        # General updates.
        protocol.NOTIFY_PANEL_BATTERY_TROUBLE: {'type': UPDATE_GENERAL, 'battery_trouble': True},
        protocol.NOTIFY_PANEL_BATTERY_RESTORED: {'type': UPDATE_GENERAL, 'battery_trouble': False},
        protocol.NOTIFY_PANEL_AC_TROUBLE: {'type': UPDATE_GENERAL, 'ac_trouble': True},
        protocol.NOTIFY_PANEL_AC_RESTORED: {'type': UPDATE_GENERAL, 'ac_trouble': False},
    }

    def __init__(self, serial_port, loop=None):
        """
        Constructs the DSC IT-100 driver.

        :param serial_port: Serial port where the IT-100 is connected to
        :param loop: Optional event loop to use
        """

        if loop is None:
            loop = asyncio.get_event_loop()

        self._loop = loop
        self._port = serial_port
        self._reader = None
        self._writer = None
        self._code = None
        self._state = state.AlarmState()

        # Handlers.
        self._handler_zone_update = None
        self._handler_partition_update = None
        self._handler_general_update = None

    @property
    def handler_zone_update(self):
        return self._handler_zone_update

    @handler_zone_update.setter
    def handler_zone_update(self, value):
        self._handler_zone_update = value

    @property
    def handler_partition_update(self):
        return self._handler_partition_update

    @handler_partition_update.setter
    def handler_partition_update(self, value):
        self._handler_partition_update = value

    @property
    def handler_general_update(self):
        return self._handler_general_update

    @handler_general_update.setter
    def handler_general_update(self, value):
        self._handler_general_update = value

    def set_alarm_code(self, code):
        """
        Configure alarm code to be sent to the IT-100 module upon request.
        """

        self._code = code

    def connect(self):
        """
        Establishes a connection with the DSC IT-100 module.
        """

        if self._reader or self._writer:
            raise exceptions.DriverError('Driver is already connected')

        ensure_future(self._connect(), loop=self._loop)

    @asyncio.coroutine
    def _connect(self):
        self._reader, self._writer = yield from serial_asyncio.open_serial_connection(
            loop=self._loop, url=self._port, baudrate=9600)

        # Spawn coroutine for handling protocol messages.
        ensure_future(self._handle_messages(), loop=self._loop)
        self.send_status_request()

    def send_message(self, command, data=''):
        """
        Sends a protocol message to the IT-100 unit.
        """

        self._writer.write(protocol.Message(command, data).serialize().encode('ascii'))

    def send_status_request(self):
        """
        Send a status request message to the IT-100 unit.
        """

        self.send_message(protocol.COMMAND_STATUS_REQUEST)

    @asyncio.coroutine
    def _handle_messages(self):
        """
        Incoming message handler.
        """

        while not self._reader.at_eof():
            line = yield from self._reader.readline()
            try:
                message = protocol.Message.deserialize(line.decode())
            except ValueError:
                logger.warning("Received malformed message from DSC IT-100.")
                continue

            if message.command == protocol.NOTIFY_CODE_REQUIRED:
                if not self._code:
                    logger.warning("Received code request, but no code has been set.")
                    continue

                self.send_message(protocol.COMMAND_CODE_SEND, self._code[:6])
            else:
                # Check if a handler is registered for the given message.
                handler = self.state_update_handlers.get(message.command, None)
                if handler is None:
                    logger.debug("No handler for notification '{}'.".format(message.command))
                    continue

                handler = handler.copy()
                handler_type = handler['type']
                del handler['type']

                if handler_type == Driver.UPDATE_ZONE:
                    # Zone update.
                    try:
                        zone = int(message.data[1:])
                    except ValueError:
                        logger.warning("Received malformed zone identifier from DSC IT-100.")
                        continue

                    zone = self._state.update_zone(zone, handler)
                    if self._handler_zone_update is not None:
                        self._handler_zone_update(self, zone)
                elif handler_type == Driver.UPDATE_PARTITION:
                    # Partition update.
                    try:
                        partition = int(message.data[0])
                    except ValueError:
                        logger.warning("Received malformed partition identifier from DSC IT-100.")
                        continue

                    partition = self._state.update_partition(partition, handler)
                    if self._handler_partition_update is not None:
                        self._handler_partition_update(self, partition)
                elif handler_type == Driver.UPDATE_GENERAL:
                    # General status update.
                    general = self._state.update_general(handler)
                    if self._handler_general_update is not None:
                        self._handler_general_update(self, general)

    def close(self):
        """
        Closes the connection with the DSC IT-100 module.
        """

        if not self._writer:
            raise exceptions.DriverError('Driver not connected')

        self._writer.transport.close()

    def get_alarm_state(self):
        """
        Return current alarm state.
        """

        return self._state



import itertools

# Command codes.
COMMAND_POLL = '000'
COMMAND_STATUS_REQUEST = '001'
COMMAND_LABELS_REQUEST = '002'
COMMAND_SET_TIME_DATE = '010'
COMMAND_OUTPUT_CONTROL = '020'
COMMAND_PARTITION_ARM_CONTROL_AWAY = '030'
COMMAND_PARTITION_ARM_CONTROL_STAY = '031'
COMMAND_PARTITION_ARM_CONTROL_ARMED = '032'
COMMAND_PARTITION_ARM_CONTROL_WITH_CODE = '033'
COMMAND_PARTITION_DISARM_CONTROL = '040'
COMMAND_TIME_STAMP_CONTROL = '055'
COMMAND_TIME_DATE_BCAST_CONTROL = '056'
COMMAND_TEMPERATURE_BCAST_CONTROL = '057'
COMMAND_VIRTUAL_KEYBOARD_CONTORL = '058'
COMMAND_TRIGGER_PANIC_ALARM = '060'
COMMAND_KEY_PRESSED = '070'
COMMAND_SET_BAUD_RATE = '080'
COMMAND_CODE_SEND = '200'

# Notification codes.
NOTIFY_ACK = '500'
NOTIFY_ERROR = '501'
NOTIFY_SYSTEM_ERROR = '502'
NOTIFY_TIME_DATE_BCAST = '550'
NOTIFY_LABELS = '570'
NOTIFY_BAUD_RATE_SET = '580'
NOTIFY_ZONE_ALARM = '601'
NOTIFY_ZONE_ALARM_RESTORE = '602'
NOTIFY_ZONE_TAMPER = '603'
NOTIFY_ZONE_TAMPER_RESTORE = '604'
NOTIFY_ZONE_FAULT = '605'
NOTIFY_ZONE_FAULT_RESTORE = '606'
NOTIFY_ZONE_OPEN = '609'
NOTIFY_ZONE_RESTORED = '610'
NOTIFY_DURESS_ALARM = '620'
NOTIFY_FIRE_KEY_ALARM = '621'
NOTIFY_FIRE_KEY_RESTORED = '622'
NOTIFY_AUXILARY_KEY_ALARM = '623'
NOTIFY_AUXILARY_KEY_RESTORED = '624'
NOTIFY_PANIC_KEY_ALARM = '625'
NOTIFY_PANIC_KEY_RESTORED = '626'
NOTIFY_AUXILARY_INPUT_ALARM = '631'
NOTIFY_AUXILARY_INPUT_RESTORED = '632'
NOTIFY_PARTITION_READY = '650'
NOTIFY_PARTITION_NOT_READY = '651'
NOTIFY_PARTITION_ARMED = '652'
NOTIFY_PARTITION_READY_TO_FORCE_ARM = '653'
NOTIFY_PARTITION_IN_ALARM = '654'
NOTIFY_PARTITION_DISARMED = '655'
NOTIFY_PARTITION_EXIT_DELAY = '656'
NOTIFY_PARTITION_ENTRY_DELAY = '657'
NOTIFY_KEYPAD_LOCKOUT = '658'
NOTIFY_KEYPAD_BLANKING = '659'
NOTIFY_COMMAND_OUTPUT = '660'
NOTIFY_INVALID_CODE = '670'
NOTIFY_FUNCTION_NOT_AVAILABLE = '671'
NOTIFY_FAILED_TO_ARM = '672'
NOTIFY_PARTITION_BUSY = '673'
NOTIFY_PARTITION_USER_CLOSING = '700'
NOTIFY_PARTITION_SPECIAL_CLOSING = '701'
NOTIFY_PARTITION_PARTIAL_CLOSING = '702'
NOTIFY_PARTITION_USER_OPENING = '750'
NOTIFY_PARTITION_SPECIAL_OPENING = '751'
NOTIFY_PANEL_BATTERY_TROUBLE = '800'
NOTIFY_PANEL_BATTERY_RESTORED = '801'
NOTIFY_PANEL_AC_TROUBLE = '802'
NOTIFY_PANEL_AC_RESTORED = '803'
NOTIFY_SYSTEM_BELL_TROUBLE = '806'
NOTIFY_SYSTEM_BELL_RESTORED = '807'
NOTIFY_GENERAL_DEV_LOW_BATTERY = '821'
NOTIFY_GENERAL_DEV_LOW_BATTERY_RESTORED = '822'
NOTIFY_GENERAL_SYSTEM_TAMPER = '829'
NOTIFY_GENERAL_SYSTEM_TAMPER_RESTORED = '830'
NOTIFY_PARTITION_TROUBLE = '840'
NOTIFY_PARTITION_TROUBLE_RESTORED = '841'
NOTIFY_FIRE_TROUBLE_ALARM = '842'
NOTIFY_FIRE_TROUBLE_RESTORED = '843'
NOTIFY_KEYBUS_FAULT = '896'
NOTIFY_KEYBUS_RESTORED = '897'
NOTIFY_CODE_REQUIRED = '900'
NOTIFY_BEEP_STATUS = '904'
NOTIFY_VERSION = '908'


class Message(object):
    """
    DSC IT-100 protocol message.
    """

    def __init__(self, command, data=''):
        """
        Constructs a DSC IT-100 protocol message.

        :param command: Command
        :param data: Data
        """

        self.command = command
        self.data = data

    def __repr__(self):
        return '<Message command="{}" data="{}">'.format(self.command, self.data)

    def serialize(self):
        """
        Serializes this message.
        """

        return ''.join([self.command, self.data, self.checksum(), '\r\n'])

    def checksum(self):
        """
        Computes and returns the checksum for this message.
        """

        return '{0:02X}'.format(sum([ord(x) for x in itertools.chain(self.command, self.data)]) % 256)

    @classmethod
    def deserialize(cls, raw):
        """
        Deserializes the message.
        """

        command = raw[0:3]
        data = raw[3:-4]
        checksum = raw[-4:-2]

        message = cls(command, data)
        if message.checksum() != checksum:
            raise ValueError('Checksum mismatch')

        return message



#!/usr/bin/env python

import os

from setuptools import setup, find_packages

VERSION = '0.1.1'

if __name__ == '__main__':
    setup(
        name='dsc-it100',
        version=VERSION,
        description="A Python driver for the DSC IT-100 integration module.",
        long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
        author='Jernej Kos',
        author_email='jernej@kos.mx',
        url='https://github.com/kostko/dsc-it100',
        license='AGPLv3',
        packages=find_packages(exclude=('*.tests', '*.tests.*', 'tests.*', 'tests')),
        package_data={},
        classifiers=[
            'Development Status :: 4 - Beta',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: GNU Affero General Public License v3',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
        ],
        include_package_data=True,
        zip_safe=False,
        install_requires=[
            'pyserial>=3.2.1',
            'pyserial-asyncio>=0.2',
        ],
        tests_require=[
        ],
        test_suite='tests',
    )
